@@ -2,6 +2,8 @@
  * (C) 2011 by On-Waves e.h.f
  * All Rights Reserved
  *
+ * SPDX-License-Identifier: GPL-2.0+
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
@@ -112,7 +114,8 @@ static void ortp_sig_cb_ssrc(RtpSession *rs, void *data)
 	uint32_t ssrc = rtp_session_get_recv_ssrc(rs);
 
 	LOGP(DLMIB, LOGL_INFO,
-	     "osmo-ortp(%d): ssrc_changed to 0x%08x\n", port, ssrc);
+	     "osmo-ortp(%d): ssrc_changed to 0x%08x, resetting\n", port, ssrc);
+	rtp_session_reset(rs);
 }
 
 static void ortp_sig_cb_pt(RtpSession *rs, void *data)
@@ -368,6 +371,8 @@ struct osmo_rtp_socket *osmo_rtp_socket_create(void *talloc_ctx, unsigned int fl
 	rtp_session_set_seq_number(rs->sess, random());
 	rs->tx_timestamp = random();
 
+	/* Make sure ssrc changes are detected immediately */
+	rtp_session_set_ssrc_changed_threshold(rs->sess, 0);
 
 	return rs;
 }
@@ -653,3 +658,10 @@ void osmo_rtp_socket_stats(struct osmo_rtp_socket *rs,
 	if (jitter)
 		*last_jitter = jitter->jitter;
 }
+
+void osmo_rtp_set_source_desc(struct osmo_rtp_socket *rs, const char *cname,
+				const char *name, const char *email, const char *phone,
+				const char *loc, const char *tool, const char *note)
+{
+	rtp_session_set_source_description(rs->sess, cname, name, email, phone, loc, tool, note);
+}
